@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2010-2020 GEM Foundation
+# Copyright (C) 2010-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -35,7 +35,7 @@ parallel, and then combine together the results. This is known as a
 of counting the letters in a text, by using the following `count`
 function:
 
-.. python::
+.. code-block:: python
 
   def count(word):
       return collections.Counter(word)
@@ -195,7 +195,9 @@ import traceback
 import collections
 from unittest import mock
 import multiprocessing.dummy
+import subprocess
 import psutil
+import getpass
 import numpy
 try:
     from setproctitle import setproctitle
@@ -203,7 +205,7 @@ except ImportError:
     def setproctitle(title):
         "Do nothing"
 
-from openquake.baselib import config, hdf5, workerpool, __version__
+from openquake.baselib import config, hdf5, workerpool, version
 from openquake.baselib.zeromq import zmq, Socket
 from openquake.baselib.performance import (
     Monitor, memory_rss, init_performance)
@@ -215,12 +217,6 @@ sys.setrecursionlimit(1200)  # raised a bit to make pickle happier
 # see https://github.com/gem/oq-engine/issues/5230
 submit = CallableDict()
 GB = 1024 ** 3
-# use only the "visible" cores, not the total system cores
-# if the underlying OS supports it (macOS does not)
-try:
-    CT = len(psutil.Process().cpu_affinity()) * 2
-except AttributeError:
-    CT = psutil.cpu_count() * 2
 
 
 @submit.add('no')
@@ -249,7 +245,7 @@ def celery_submit(self, func, args, monitor):
 def zmq_submit(self, func, args, monitor):
     if not hasattr(self, 'sender'):
         port = int(config.zworkers.ctrl_port) + 2
-        task_input_url = 'tcp://127.0.0.1:%d' % port
+        task_input_url = 'tcp://0.0.0.0:%d' % port
         self.sender = Socket(
             task_input_url, zmq.PUSH, 'connect').__enter__()
     return self.sender.send((func, args, self.task_no, monitor))
@@ -257,14 +253,15 @@ def zmq_submit(self, func, args, monitor):
 
 @submit.add('dask')
 def dask_submit(self, func, args, monitor):
-    return self.dask_client.submit(safely_call, func, args, self.task_no)
+    return self.dask_client.submit(
+        safely_call, func, args, self.task_no, monitor)
 
 
 def oq_distribute(task=None):
     """
-    :returns: the value of OQ_DISTRIBUTE or 'processpool'
+    :returns: the value of OQ_DISTRIBUTE or config.distribution.oq_distribute
     """
-    dist = os.environ.get('OQ_DISTRIBUTE', 'processpool').lower()
+    dist = os.environ.get('OQ_DISTRIBUTE', config.distribution.oq_distribute)
     if dist not in ('no', 'processpool', 'threadpool', 'celery', 'zmq',
                     'dask'):
         raise ValueError('Invalid oq_distribute=%s' % dist)
@@ -384,6 +381,7 @@ class Result(object):
         self.mon = mon
         self.tb_str = tb_str
         self.msg = msg
+        self.workerid = (socket.gethostname(), os.getpid())
 
     def get(self):
         """
@@ -409,11 +407,11 @@ class Result(object):
         :returns: a new Result instance
         """
         try:
-            if mon.version != __version__:
+            if mon.version != version:
                 raise RuntimeError(
                     'The master is at version %s while the worker %s is at '
                     'version %s' % (mon.version, socket.gethostname(),
-                                    __version__))
+                                    version))
             with mon:
                 val = func(*args)
         except StopIteration:
@@ -445,7 +443,7 @@ def check_mem_usage(soft_percent=None, hard_percent=None):
 
 
 dummy_mon = Monitor()
-dummy_mon.version = __version__
+dummy_mon.version = version
 dummy_mon.backurl = None
 
 
@@ -565,7 +563,6 @@ class IterResult(object):
                 name = result.mon.operation[6:]  # strip 'total '
                 result.mon.save_task_info(self.h5, result, name, mem_gb)
                 result.mon.flush(self.h5)
-                self.h5.flush()
             elif not result.func:  # real output
                 yield val
 
@@ -622,9 +619,11 @@ def init_workers():
 
 
 def getargnames(task_func):
-    # a task can be a function, a class or an instance with a __call__
+    # a task can be a function, a method, a class or a callable instance
     if inspect.isfunction(task_func):
         return inspect.getfullargspec(task_func).args
+    elif inspect.ismethod(task_func):
+        return inspect.getfullargspec(task_func).args[1:]
     elif inspect.isclass(task_func):
         return inspect.getfullargspec(task_func.__init__).args[1:]
     else:  # instance with a __call__ method
@@ -634,12 +633,18 @@ def getargnames(task_func):
 class Starmap(object):
     pids = ()
     running_tasks = []  # currently running tasks
-    # use only the "visible" cores, not the total system cores
-    # if the underlying OS supports it (macOS does not)
-    num_cores = None
+    num_cores = int(config.distribution.get('num_cores', '0'))
+    if not num_cores:
+        # use only the "visible" cores, not the total system cores
+        # if the underlying OS supports it (macOS does not)
+        try:
+            num_cores = len(psutil.Process().cpu_affinity())
+        except AttributeError:
+            num_cores = psutil.cpu_count()
+    CT = num_cores * 2
 
     @classmethod
-    def init(cls, poolsize=None, distribute=None):
+    def init(cls, distribute=None):
         cls.distribute = distribute or oq_distribute()
         if cls.distribute == 'processpool' and not hasattr(cls, 'pool'):
             # unregister custom handlers before starting the processpool
@@ -649,14 +654,14 @@ class Starmap(object):
             # https://github.com/gem/oq-engine/pull/3923 and
             # https://codewithoutrules.com/2018/09/04/python-multiprocessing/
             cls.pool = multiprocessing.get_context('spawn').Pool(
-                poolsize, init_workers)
+                cls.num_cores, init_workers)
             # after spawning the processes restore the original handlers
             # i.e. the ones defined in openquake.engine.engine
             signal.signal(signal.SIGTERM, term_handler)
             signal.signal(signal.SIGINT, int_handler)
             cls.pids = [proc.pid for proc in cls.pool._pool]
         elif cls.distribute == 'threadpool' and not hasattr(cls, 'pool'):
-            cls.pool = multiprocessing.dummy.Pool(poolsize)
+            cls.pool = multiprocessing.dummy.Pool(cls.num_cores)
         elif cls.distribute == 'dask':
             cls.dask_client = Client(config.distribution.dask_scheduler)
 
@@ -677,8 +682,7 @@ class Starmap(object):
     def apply(cls, task, args, concurrent_tasks=None,
               maxweight=None, weight=lambda item: 1,
               key=lambda item: 'Unspecified',
-              distribute=None, progress=logging.info, h5=None,
-              num_cores=None):
+              distribute=None, progress=logging.info, h5=None):
         r"""
         Apply a task to a tuple of the form (sequence, \*other_args)
         by first splitting the sequence in chunks, according to the weight
@@ -694,7 +698,6 @@ class Starmap(object):
         :param distribute: if not given, inferred from OQ_DISTRIBUTE
         :param progress: logging function to use (default logging.info)
         :param h5: an open hdf5.File where to store the performance info
-        :param num_cores: the number of available cores
         :returns: an :class:`IterResult` object
         """
         arg0, *args = args
@@ -703,15 +706,13 @@ class Starmap(object):
                 arg0, maxweight, weight, key))
         else:  # split_in_blocks is eager
             if concurrent_tasks is None:
-                concurrent_tasks = CT
+                concurrent_tasks = cls.CT
             taskargs = [[blk] + args for blk in split_in_blocks(
                 arg0, concurrent_tasks or 1, weight, key)]
-        return cls(
-            task, taskargs, distribute, progress, h5, num_cores
-        ).submit_all()
+        return cls(task, taskargs, distribute, progress, h5)
 
     def __init__(self, task_func, task_args=(), distribute=None,
-                 progress=logging.info, h5=None, num_cores=None):
+                 progress=logging.info, h5=None):
         self.__class__.init(distribute=distribute)
         self.task_func = task_func
         if h5:
@@ -722,12 +723,12 @@ class Starmap(object):
             h5 = hdf5.File(gettemp(suffix='.hdf5'), 'w')
             init_performance(h5)
         self.monitor = Monitor(task_func.__name__)
+        self.monitor.filename = h5.filename
         self.monitor.calc_id = self.calc_id
         self.name = self.monitor.operation or task_func.__name__
         self.task_args = task_args
         self.progress = progress
         self.h5 = h5
-        self.num_cores = num_cores
         self.task_queue = []
         try:
             self.num_tasks = len(self.task_args)
@@ -742,10 +743,12 @@ class Starmap(object):
         self.monitor.backurl = None  # overridden later
         self.tasks = []  # populated by .submit
         self.task_no = 0
-        if self.distribute == 'zmq':  # add a check
-            err = workerpool.check_status()
-            if err:
-                raise RuntimeError(err)
+        self.t0 = time.time()
+        if self.distribute in 'zmq dask celery':  # add a check
+            errors = ['The workerpool on %s is down' % host
+                      for host, run, tot in workers_status() if tot == 0]
+            if errors:
+                raise RuntimeError('\n'.join(errors))
 
     def log_percent(self):
         """
@@ -756,12 +759,8 @@ class Starmap(object):
         total = submitted + queued
         done = submitted - self.todo
         percent = int(float(done) / total * 100)
-        fname = self.task_func.__name__
         if not hasattr(self, 'prev_percent'):  # first time
             self.prev_percent = 0
-            nbytes = sum(self.sent[fname].values())
-            self.progress('%s %s sent, %d submitted, %d queued',
-                          self.name, humansize(nbytes), submitted, queued)
         elif percent > self.prev_percent:
             self.progress('%s %3d%% [%d submitted, %d queued]',
                           self.name, percent, submitted, queued)
@@ -775,11 +774,12 @@ class Starmap(object):
         monitor = monitor or self.monitor
         func = func or self.task_func
         if not hasattr(self, 'socket'):  # first time
+            self.t0 = time.time()
             self.__class__.running_tasks = self.tasks
             self.socket = Socket(self.receiver, zmq.PULL, 'bind').__enter__()
             monitor.backurl = 'tcp://%s:%s' % (
                 config.dbserver.host, self.socket.port)
-            monitor.version = __version__
+            monitor.version = version
         OQ_TASK_NO = os.environ.get('OQ_TASK_NO')
         if OQ_TASK_NO is not None and self.task_no != int(OQ_TASK_NO):
             self.task_no += 1
@@ -839,15 +839,20 @@ class Starmap(object):
                 self.todo += 1
 
     def _loop(self):
-        num_cores = self.num_cores or CT // 2
+        self.busytime = AccumDict(accum=[])  # pid -> time
         if self.task_queue:
-            first_args = self.task_queue[:num_cores]
-            self.task_queue[:] = self.task_queue[num_cores:]
+            first_args = self.task_queue[:self.num_cores]
+            self.task_queue[:] = self.task_queue[self.num_cores:]
             for func, args in first_args:
                 self.submit(args, func=func)
         if not hasattr(self, 'socket'):  # no submit was ever made
             return ()
 
+        nbytes = sum(self.sent[self.task_func.__name__].values())
+        if nbytes > 1E6:
+            logging.info('Sent %d tasks, %s in %d seconds', len(self.tasks),
+                         humansize(nbytes), time.time() - self.t0)
+
         isocket = iter(self.socket)
         self.todo = len(self.tasks)
         while self.todo:
@@ -857,25 +862,28 @@ class Starmap(object):
                 logging.warning('Discarding a result from job %s, since this '
                                 'is job %d', res.mon.calc_id, self.calc_id)
             elif res.msg == 'TASK_ENDED':
+                self.busytime += {res.workerid: res.mon.duration}
                 self.todo -= 1
                 self._submit_many(1)
-                logging.debug('%d tasks todo, %d in queue',
+                logging.debug('%d tasks running, %d in queue',
                               self.todo, len(self.task_queue))
                 yield res
             elif res.func:  # add subtask
                 self.task_queue.append((res.func, res.pik))
-                if self.num_cores is None:
-                    self._submit_many(1)  # oversubmit
-                elif self.todo < self.num_cores:
-                    self._submit_many(self.num_cores - self.todo)
+                self._submit_many(1)
             else:
                 yield res
         self.log_percent()
         self.socket.__exit__(None, None, None)
         self.tasks.clear()
+        if len(self.busytime) > 1:
+            times = numpy.array(list(self.busytime.values()))
+            logging.info(
+                'Mean time per core=%ds, std=%.1fs, min=%ds, max=%ds',
+                times.mean(), times.std(), times.min(), times.max())
 
 
-def sequential_apply(task, args, concurrent_tasks=CT,
+def sequential_apply(task, args, concurrent_tasks=Starmap.CT,
                      maxweight=None, weight=lambda item: 1,
                      key=lambda item: 'Unspecified',
                      progress=logging.info):
@@ -920,3 +928,120 @@ def split_task(func, *args, duration=1000,
     for block in blocks[:-1]:
         yield (func, block) + args[1:-1]
     yield func(*(blocks[-1],) + args[1:])
+
+#                             start/stop workers                             #
+
+
+OQDIST = oq_distribute()
+
+
+def ssh_args():
+    remote_python = config.zworkers.remote_python or sys.executable
+    remote_user = config.zworkers.remote_user or getpass.getuser()
+    if config.zworkers.host_cores.strip():
+        for hostcores in config.zworkers.host_cores.split(','):
+            host, cores = hostcores.split()
+            if host == '127.0.0.1':  # localhost
+                yield host, cores, [sys.executable]
+            else:
+                yield host, cores, [
+                    'ssh', '-f', '-T', remote_user + '@' + host, remote_python]
+
+
+def workers_start():
+    """
+    Start the remote workers with ssh
+    """
+    if OQDIST in 'no processpool':
+        return
+    sched = config.distribution.dask_scheduler
+    for host, cores, args in ssh_args():
+        if OQDIST == 'dask':
+            args += ['-m', 'distributed.cli.dask_worker', sched,
+                     '--nprocs', cores, '--nthreads', '1',
+                     '--memory-limit', '1e11']
+        elif OQDIST == 'celery':
+            args += ['-m', 'celery', 'worker', '--purge', '-O', 'fair',
+                     '--config', 'openquake.engine.celeryconfig']
+            if cores != '-1':
+                args += ['-c', cores]
+        elif OQDIST == 'zmq':
+            args += ['-m', 'openquake.baselib.workerpool', '-n', cores]
+        subprocess.Popen(args, start_new_session=True)
+        logging.info(args)
+
+
+def workers_stop():
+    """
+    Stop all the workers with a shutdown
+    """
+    if OQDIST == 'dask':
+        Client(config.distribution.dask_scheduler).retire_workers()
+    elif OQDIST == 'celery':
+        app.control.shutdown()
+    elif OQDIST == 'zmq':
+        workerpool.WorkerMaster().kill()
+    return 'stopped'
+
+
+def workers_status():
+    """
+    :returns: a list [(host name, running, total), ...]
+    """
+    if OQDIST == 'dask':
+        with Client(config.distribution.dask_scheduler) as c:
+            info = c.scheduler_info()
+        acc = AccumDict(accum=numpy.zeros(2, int))  # IP -> (running, total)
+        for uri, worker in info['workers'].items():
+            ip = uri.split(':')[1]  # 'tcp://192.168.2.2:3429' => //192.168.2.2
+            ex = bool(worker['metrics']['executing'])
+            acc[ip[2:]] += numpy.array([ex, 1])
+        return [(host, arr[0], arr[1]) for host, arr in acc.items()]
+
+    elif OQDIST == 'celery':
+        stats = app.control.inspect(timeout=1).stats() or {}
+        out = []
+        for host, worker in stats.items():
+            total = worker['pool']['max-concurrency']
+            out.append((host, total, total))
+        return out
+
+    elif OQDIST == 'zmq':
+        return workerpool.WorkerMaster().status()
+
+
+def workers_wait(seconds=30):
+    """
+    Wait until all workers are active
+    """
+    if OQDIST in 'dask celery zmq':
+        num_hosts = len(config.zworkers.host_cores.split(','))
+        for _ in range(seconds):
+            time.sleep(1)
+            status = workers_status()
+            if len(status) == num_hosts and all(
+                    total for host, running, total in status):
+                break
+        else:
+            raise TimeoutError(status)
+        return status
+
+
+def workers_kill():
+    code = '''import getpass, psutil
+user = getpass.getuser()
+for proc in psutil.process_iter(['name', 'username']):
+    if proc.username() == user:
+        cmdline = proc.cmdline()
+        if ('workerpool' in cmdline or 'celery' in cmdline or
+            'distributed.cli.dask_worker' in cmdline or
+            'distributed.cli.dask_scheduler' in cmdline):
+            print('killing %s' % ' '.join(cmdline))
+            proc.kill()
+'''
+    hosts = []
+    for host, cores, args in ssh_args():
+        c = '"%s"' % code if 'ssh' in args else code
+        out = subprocess.check_output(args + ['-c', c]).decode('utf8')
+        hosts.append('%s: %s' % (host, out))
+    return '\n'.join(hosts)
