@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2017-2020 GEM Foundation
+# Copyright (C) 2017-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -17,6 +17,7 @@
 # along with OpenQuake.  If not, see <http://www.gnu.org/licenses/>.
 import re
 import zmq
+import time
 import logging
 
 context = zmq.Context()
@@ -73,9 +74,9 @@ class Socket(object):
     :param end_point: zmq end point string
     :param socket_type: zmq socket type (integer)
     :param mode: default 'bind', accepts also 'connect'
-    :param timeout: default 5000 ms, used when polling the underlying socket
+    :param timeout: default 15000 ms, used when polling the underlying socket
     """
-    def __init__(self, end_point, socket_type, mode, timeout=5000):
+    def __init__(self, end_point, socket_type, mode, timeout=15000):
         assert socket_type in (zmq.REP, zmq.REQ, zmq.PULL, zmq.PUSH)
         assert mode in ('bind', 'connect'), mode
         if mode == 'bind':
@@ -95,8 +96,15 @@ class Socket(object):
             p1, p2 = map(int, port_range.groups())
             end_point = self.end_point.rsplit(':', 1)[0]  # strip port range
             self.zsocket = context.socket(self.socket_type)
-            port = self.zsocket.bind_to_random_port(end_point, p1, p2)
-            # NB: will raise a ZMQBindError if no port is available
+            while True:
+                try:
+                    # NB: will raise a ZMQBindError if no port is available
+                    port = self.zsocket.bind_to_random_port(end_point, p1, p2)
+                except zmq.error.ZMQBindError:
+                    logging.info('Waiting for a zmq port')
+                    time.sleep(30)
+                else:
+                    break
             self.port = port
         elif self.mode == 'bind':
             self.zsocket = bind(self.end_point, self.socket_type)
