@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2018-2020 GEM Foundation
+# Copyright (C) 2018-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -19,10 +19,10 @@ import sys
 import runpy
 from functools import partial
 import numpy
-from openquake.baselib import sap
 from openquake.hazardlib import nrml
 from openquake.baselib.datastore import read
 from openquake.hazardlib.geo.geodetic import geodetic_distance
+from openquake.hazardlib.contexts import Timer
 from openquake.commonlib import readinput, calc, logs
 from openquake.calculators.base import get_calc
 from openquake.calculators.extract import extract, WebExtractor
@@ -52,6 +52,7 @@ class OpenQuake(object):
         self.get_calc = lambda job_ini: get_calc(job_ini, logs.init())
         self.make_hmap = calc.make_hmap
         self.geodetic_distance = geodetic_distance
+        self.Timer = Timer
         # TODO: more utilities will be added when deemed useful
 
     def webex(self, calc_id, what):
@@ -70,9 +71,16 @@ class OpenQuake(object):
         finally:
             ex.close()
 
+    def read_ruptures(self, calc_id, field):
+        dstore = read(calc_id)
+        lst = []
+        for name, dset in dstore.items():
+            if name.startswith('rup_'):
+                lst.append(dset[field][:])
+        return numpy.concatenate(lst)
 
-@sap.script
-def shell(script=None, args=()):
+
+def main(script=None, args=()):
     """
     Start an embedded (i)python instance with a global object "o" or
     run a Python script in the engine environment.
@@ -91,5 +99,5 @@ def shell(script=None, args=()):
                       local=dict(o=o))
 
 
-shell.arg('script', 'python script to run (if any)')
-shell.arg('args', 'arguments to pass to the script', nargs='*')
+main.script = 'python script to run (if any)'
+main.args = dict(help='arguments to pass to the script', nargs='*')
