@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2020 GEM Foundation
+# Copyright (C) 2012-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -47,7 +47,7 @@ class BBoxError(ValueError):
     """Bounding box too large"""
 
 
-def angular_distance(km, lat, lat2=None):
+def angular_distance(km, lat=0, lat2=None):
     """
     Return the angular distance of two points at the given latitude.
 
@@ -344,6 +344,11 @@ def get_spherical_bounding_box(lons, lats):
         180 degrees (it is impossible to define a single hemisphere
         bound to poles that would contain the whole collection).
     """
+    ok = numpy.isfinite(lons)
+    if not ok.all():
+        lons = lons[ok]
+        lats = lats[ok]
+
     north, south = numpy.max(lats), numpy.min(lats)
     west, east = numpy.min(lons), numpy.max(lons)
     assert (-180 <= west <= 180) and (-180 <= east <= 180), (west, east)
@@ -637,7 +642,8 @@ def bbox2poly(bbox):
 # geohash code adapted from Leonard Norrgard's implementation
 # https://github.com/vinsci/geohash/blob/master/Geohash/geohash.py
 # see also https://en.wikipedia.org/wiki/Geohash
-# length 5 = 2.4 km resolution, length 4 = 20 km, length 3 = 78 km
+# length 6 = .61 km  resolution, length 5 = 2.4 km resolution,
+# length 4 = 20 km, length 3 = 78 km
 # it may turn useful in the future (with SiteCollection.geohash)
 def geohash(lon, lat, length):
     """
