@@ -1,5 +1,5 @@
 # The Hazard Library
-# Copyright (C) 2012-2020 GEM Foundation
+# Copyright (C) 2012-2021 GEM Foundation
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -18,10 +18,13 @@ Module :mod:`openquake.hazardlib.source.base` defines a base class for
 seismic sources.
 """
 import abc
+import zlib
 import numpy
 from openquake.hazardlib.geo import Point
 from openquake.hazardlib.source.rupture import ParametricProbabilisticRupture
 
+EPS = .01  # used for src.nsites outside the maximum_distance
+
 
 class BaseSeismicSource(metaclass=abc.ABCMeta):
     """
@@ -36,12 +39,12 @@ class BaseSeismicSource(metaclass=abc.ABCMeta):
     :param tectonic_region_type:
         Source's tectonic regime. See :class:`openquake.hazardlib.const.TRT`.
     """
+    et_id = 0  # set by the engine
+    nsites = 0  # set when filtering the source
     ngsims = 1
     min_mag = 0  # set in get_oqparams and CompositeSourceModel.filter
     splittable = True
-    serial = 0  # set in init_serials
     checksum = 0  # set in source_reader
-    grp_id = ()
 
     @abc.abstractproperty
     def MODIFICATIONS(self):
@@ -52,44 +55,45 @@ class BaseSeismicSource(metaclass=abc.ABCMeta):
         """
         Determine the source weight from the number of ruptures
         """
+        # NB: for point sources .num_ruptures is preset in preclassical,
+        # and it is less than the real number of ruptures if the
+        # pointsource_distance is set
         if not self.num_ruptures:
             self.num_ruptures = self.count_ruptures()
-        if hasattr(self, 'nodal_plane_distribution'):
-            rescale = len(self.nodal_plane_distribution.data) * len(
-                self.hypocenter_distribution.data)
-            return self.num_ruptures / rescale
-        return self.num_ruptures
+        w = self.num_ruptures * self.ngsims * (.1 if self.nsites == EPS else 1)
+        if not hasattr(self, 'nodal_plane_distribution'):  # not pointlike
+            w *= 10  # increase weight of non point sources
+        return w
 
     @property
-    def nsites(self):
-        """
-        :returns: the number of sites affected by this source
-        """
-        try:
-            # the engine sets self.indices when filtering the sources
-            return len(self.indices)
-        except AttributeError:
-            # this happens in several hazardlib tests, therefore we return
-            # a fake number of affected sites to avoid changing all tests
-            return 1
-
-    @property
-    def grp_ids(self):
+    def et_ids(self):
         """
         :returns: a list of source group IDs (usually of 1 element)
         """
-        grp_id = self.grp_id
-        return [grp_id] if isinstance(grp_id, int) else grp_id
+        et_id = self.et_id
+        return [et_id] if isinstance(et_id, int) else et_id
+
+    def serial(self, ses_seed):
+        """
+        :returns: a random seed derived from source_id and ses_seed
+        """
+        return zlib.crc32(self.source_id.encode('ascii'), ses_seed)
 
     def __init__(self, source_id, name, tectonic_region_type):
         self.source_id = source_id
         self.name = name
         self.tectonic_region_type = tectonic_region_type
-        self.grp_id = -1  # set by the engine
+        self.et_id = -1  # set by the engine
         self.num_ruptures = 0  # set by the engine
         self.seed = None  # set by the engine
         self.min_mag = 0  # set by the SourceConverter
 
+    def is_gridded(self):
+        """
+        :returns: True if the source contains only gridded ruptures
+        """
+        return False
+
     @abc.abstractmethod
     def iter_ruptures(self, **kwargs):
         """
@@ -101,18 +105,18 @@ class BaseSeismicSource(metaclass=abc.ABCMeta):
             `~openquake.hazardlib.source.rupture.BaseProbabilisticRupture`.
         """
 
-    def sample_ruptures(self, eff_num_ses):
+    def sample_ruptures(self, eff_num_ses, ses_seed):
         """
         :param eff_num_ses: number of stochastic event sets * number of samples
-        :yields: pairs (rupture, num_occurrences[num_samples])
+        :yields: triples (rupture, et_id, num_occurrences)
         """
-        rup_id = self.serial
-        numpy.random.seed(self.serial)
-        for grp_id in self.grp_ids:
+        seed = self.serial(ses_seed)
+        numpy.random.seed(seed)
+        for et_id in self.et_ids:
             for rup, num_occ in self._sample_ruptures(eff_num_ses):
-                rup.rup_id = rup_id
-                rup_id += 1
-                yield rup, grp_id, num_occ
+                rup.rup_id = seed
+                seed += 1
+                yield rup, et_id, num_occ
 
     def _sample_ruptures(self, eff_num_ses):
         tom = getattr(self, 'temporal_occurrence_model', None)
@@ -137,6 +141,9 @@ class BaseSeismicSource(metaclass=abc.ABCMeta):
         if hasattr(self, 'get_annual_occurrence_rates'):
             for mag, rate in self.get_annual_occurrence_rates():
                 mags.add(mag)
+        elif hasattr(self, 'source_file'):
+            # unbound UCERFSource
+            mags.add(numpy.nan)
         else:  # nonparametric
             for rup, pmf in self.data:
                 if rup.mag >= self.min_mag:
@@ -185,7 +192,7 @@ class BaseSeismicSource(metaclass=abc.ABCMeta):
                 yield rup, num_occ
 
     @abc.abstractmethod
-    def get_one_rupture(self, rupture_mutex=False):
+    def get_one_rupture(self, ses_seed, rupture_mutex=False):
         """
         Yields one random rupture from a source
         """
@@ -231,6 +238,13 @@ class BaseSeismicSource(metaclass=abc.ABCMeta):
         meth = getattr(self, 'modify_%s' % modification)
         meth(**parameters)
 
+    def to_xml(self):
+        """
+        Convert the source into an XML string, very useful for debugging
+        """
+        from openquake.hazardlib import nrml, sourcewriter
+        return nrml.to_string(sourcewriter.obj_to_node(self))
+
 
 class ParametricSeismicSource(BaseSeismicSource, metaclass=abc.ABCMeta):
     """
@@ -317,10 +331,9 @@ class ParametricSeismicSource(BaseSeismicSource, metaclass=abc.ABCMeta):
         String representation of a source, displaying the source class name
         and the source id.
         """
-        return '<%s %s (%s)>' % (
-            self.__class__.__name__, self.source_id, self.name)
+        return '<%s %s>' % (self.__class__.__name__, self.source_id)
 
-    def get_one_rupture(self, rupture_mutex=False):
+    def get_one_rupture(self, ses_seed, rupture_mutex=False):
         """
         Yields one random rupture from a source
         """
@@ -330,7 +343,8 @@ class ParametricSeismicSource(BaseSeismicSource, metaclass=abc.ABCMeta):
         assert (not rupture_mutex), msg
         # Set random seed and get the number of ruptures
         num_ruptures = self.count_ruptures()
-        numpy.random.seed(self.seed)
+        seed = self.serial(ses_seed)
+        numpy.random.seed(seed)
         idx = numpy.random.choice(num_ruptures)
         # NOTE Would be nice to have a method generating a rupture given two
         # indexes, one for magnitude and one setting the position
