@@ -1,5 +1,5 @@
 # The Hazard Library
-# Copyright (C) 2013-2020 GEM Foundation
+# Copyright (C) 2013-2021 GEM Foundation
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -18,17 +18,21 @@ Module :mod:`openquake.hazardlib.source.non_parametric` defines
 :class:`NonParametricSeismicSource`
 """
 import numpy
+
+from openquake.hazardlib.geo.surface.base import _get_finite_mesh
+
 from openquake.hazardlib.source.base import BaseSeismicSource
 from openquake.hazardlib.geo.surface.gridded import GriddedSurface
 from openquake.hazardlib.geo.surface.multi import MultiSurface
 from openquake.hazardlib.source.rupture import \
     NonParametricProbabilisticRupture
 from openquake.hazardlib.geo.utils import angular_distance, KM_TO_DEGREES
-from openquake.hazardlib.geo.mesh import Mesh, point3d
+from openquake.hazardlib.geo.mesh import Mesh
 from openquake.hazardlib.geo.point import Point
 from openquake.hazardlib.pmf import PMF
 
 F32 = numpy.float32
+U32 = numpy.uint32
 
 
 class NonParametricSeismicSource(BaseSeismicSource):
@@ -51,9 +55,14 @@ class NonParametricSeismicSource(BaseSeismicSource):
     code = b'N'
     MODIFICATIONS = set()
 
-    def __init__(self, source_id, name, tectonic_region_type, data):
+    def __init__(self, source_id, name, tectonic_region_type, data,
+                 weights=None):
         super().__init__(source_id, name, tectonic_region_type)
         self.data = data
+        if weights is not None:
+            assert len(weights) == len(data)
+            for (rup, pmf), weight in zip(data, weights):
+                rup.weight = weight
 
     def iter_ruptures(self, **kwargs):
         """
@@ -79,7 +88,8 @@ class NonParametricSeismicSource(BaseSeismicSource):
             src = self.__class__(source_id, self.name,
                                  self.tectonic_region_type, [rup_pmf])
             src.num_ruptures = 1
-            src.grp_id = self.grp_id
+            src.et_id = self.et_id
+            src.id = self.id
             yield src
 
     def count_ruptures(self):
@@ -124,35 +134,53 @@ class NonParametricSeismicSource(BaseSeismicSource):
                 return False
         return True
 
-    def __toh5__(self):
+    def todict(self):
+        """
+        Convert a GriddedSource into a dictionary of arrays
+        """
         assert self.is_gridded(), '%s is not gridded' % self
-        attrs = {'source_id': self.source_id, 'name': self.name,
-                 'tectonic_region_type': self.tectonic_region_type}
-        dic = {'probs_occur': [], 'magnitude': [], 'rake': [],
-               'hypocenter': [], 'points': []}
-        for rup, pmf in self.data:
-            dic['probs_occur'].append([prob for (prob, _) in pmf.data])
-            dic['magnitude'].append(rup.mag)
-            dic['rake'].append(rup.rake)
-            dic['hypocenter'].append((rup.hypocenter.x, rup.hypocenter.y,
-                                      rup.hypocenter.z))
-            dic['points'].append(rup.surface.mesh.array)
-        dic['hypocenter'] = numpy.array(dic['hypocenter'], point3d)
-        return dic, attrs
+        n = len(self.data)
+        m = sum(len(rup.surface.mesh) for rup, pmf in self.data)
+        p = len(self.data[0][1].data)
+        dic = {'probs_occur': numpy.zeros((n, p)),
+               'magnitude': numpy.zeros(n),
+               'rake': numpy.zeros(n),
+               'hypocenter': numpy.zeros((n, 3), F32),
+               'mesh3d': numpy.zeros((m, 3), F32),
+               'slice': numpy.zeros((n, 2), U32)}
+        start = 0
+        for i, (rup, pmf) in enumerate(self.data):
+            dic['probs_occur'][i] = [prob for (prob, _) in pmf.data]
+            dic['magnitude'][i] = rup.mag
+            dic['rake'][i] = rup.rake
+            dic['hypocenter'][i] = (rup.hypocenter.x, rup.hypocenter.y,
+                                    rup.hypocenter.z)
+            mesh = rup.surface.mesh.array.T  # shape (npoints, 3)
+            dic['mesh3d'][start: start + len(mesh)] = mesh
+            dic['slice'][i] = start, start + len(mesh)
+            start += len(mesh)
+        return dic
 
-    def __fromh5__(self, dic, attrs):
-        vars(self).update(attrs)
-        self.data = []
-        for mag, rake, hp, probs, points in zip(
+    def fromdict(self, dic, weights=None):
+        """
+        Populate a GriddedSource with ruptures
+        """
+        assert not self.data, '%s is not empty' % self
+        i = 0
+        for mag, rake, hp, probs, (start, stop) in zip(
                 dic['magnitude'], dic['rake'], dic['hypocenter'],
-                dic['probs_occur'], dic['points']):
-            mesh = Mesh(points[0], points[1], points[2])
+                dic['probs_occur'], dic['slice']):
+            mesh = Mesh(dic['mesh3d'][start:stop, 0],
+                        dic['mesh3d'][start:stop, 1],
+                        dic['mesh3d'][start:stop, 2])
             surface = GriddedSurface(mesh)
             pmf = PMF([(prob, i) for i, prob in enumerate(probs)])
-            hypocenter = Point(hp['lon'], hp['lat'], hp['depth'])
+            hypocenter = Point(hp[0], hp[1], hp[2])
             rup = NonParametricProbabilisticRupture(
-                mag, rake, self.tectonic_region_type, hypocenter, surface, pmf)
+                mag, rake, self.tectonic_region_type, hypocenter, surface, pmf,
+                weight=None if weights is None else weights[i])
             self.data.append((rup, pmf))
+            i += 1
 
     def __repr__(self):
         return '<%s gridded=%s>' % (self.__class__.__name__, self.is_gridded())
@@ -162,15 +190,27 @@ class NonParametricSeismicSource(BaseSeismicSource):
         """
         The convex hull of the underlying mesh of points
         """
-        lons = numpy.concatenate(
-            [rup.surface.mesh.lons.flatten() for rup, pmf in self.data])
-        lats = numpy.concatenate(
-            [rup.surface.mesh.lats.flatten() for rup, pmf in self.data])
+        lons, lats = [], []
+        for rup, pmf in self.data:
+
+            if isinstance(rup.surface, MultiSurface):
+                for sfc in rup.surface.surfaces:
+                    lons.extend(sfc.mesh.lons.flat)
+                    lats.extend(sfc.mesh.lats.flat)
+            else:
+                lons.extend(rup.surface.mesh.lons.flat)
+                lats.extend(rup.surface.mesh.lats.flat)
+
+        condition = numpy.isfinite(lons).astype(int)
+        lons = numpy.extract(condition, lons)
+        lats = numpy.extract(condition, lats)
+
         points = numpy.zeros(len(lons), [('lon', F32), ('lat', F32)])
-        points['lon'] = lons
-        points['lat'] = lats
+        points['lon'] = numpy.round(lons, 5)
+        points['lat'] = numpy.round(lats, 5)
         points = numpy.unique(points)
         mesh = Mesh(points['lon'], points['lat'])
+
         return mesh.get_convex_hull()
 
     def wkt(self):
@@ -179,7 +219,7 @@ class NonParametricSeismicSource(BaseSeismicSource):
         """
         return self.polygon.wkt
 
-    def get_one_rupture(self, rupture_mutex=False):
+    def get_one_rupture(self, ses_seed, rupture_mutex=False):
         """
         Yields one random rupture from a source
         """
@@ -187,10 +227,11 @@ class NonParametricSeismicSource(BaseSeismicSource):
         if rupture_mutex:
             weights = numpy.array([rup.weight for rup in self.iter_ruptures()])
         else:
-            weights = numpy.ones((num_ruptures))*1./num_ruptures
+            weights = numpy.ones((num_ruptures)) / num_ruptures
         idx = numpy.random.choice(range(num_ruptures), p=weights)
+        serial = self.serial(ses_seed)
         for i, rup in enumerate(self.iter_ruptures()):
             if i == idx:
-                rup.rup_id = self.serial + i
+                rup.rup_id = serial + i
                 rup.idx = idx
                 return rup
