@@ -1,6 +1,6 @@
 # coding: utf-8
 # The Hazard Library
-# Copyright (C) 2012-2020 GEM Foundation
+# Copyright (C) 2012-2021 GEM Foundation
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -24,80 +24,152 @@ import abc
 import numpy
 import math
 import itertools
-import toml
-from openquake.baselib import general
+import json
+from openquake.baselib import general, hdf5
 from openquake.hazardlib import geo, contexts
 from openquake.hazardlib.geo.nodalplane import NodalPlane
 from openquake.hazardlib.geo.mesh import (
-    Mesh, RectangularMesh, surface_to_array)
+    Mesh, RectangularMesh, surface_to_arrays)
 from openquake.hazardlib.geo.point import Point
 from openquake.hazardlib.geo.geodetic import geodetic_distance
 from openquake.hazardlib.near_fault import (
     get_plane_equation, projection_pp, directp, average_s_rad, isochone_ratio)
 from openquake.hazardlib.geo.surface.base import BaseSurface
 
+U8 = numpy.uint8
 U16 = numpy.uint16
 U32 = numpy.uint32
 F32 = numpy.float32
+F64 = numpy.float64
 TWO16 = 2 ** 16
 TWO32 = 2 ** 32
 pmf_dt = numpy.dtype([('prob', float), ('occ', U32)])
 events_dt = numpy.dtype([('id', U32), ('rup_id', U32), ('rlz_id', U16)])
+rupture_dt = numpy.dtype([('seed', U32),
+                          ('mag', F32),
+                          ('rake', F32),
+                          ('lon', F32),
+                          ('lat', F32),
+                          ('dep', F32),
+                          ('multiplicity', U32),
+                          ('trt', hdf5.vstr),
+                          ('kind', hdf5.vstr),
+                          ('mesh', hdf5.vstr),
+                          ('extra', hdf5.vstr)])
+
 code2cls = {}
 
 
-def _get_rupture(dic, geom=None, trt=None):
-    # dic: a dictionary or a record
-    # geom: if any, an array with fields lon, lat, depth
+def to_csv_array(ruptures):
+    """
+    :param ruptures: a list of ruptures
+    :returns: an array of ruptures suitable for serialization in CSV
+    """
+    if not code2cls:
+        code2cls.update(BaseRupture.init())
+    arr = numpy.zeros(len(ruptures), rupture_dt)
+    for rec, rup in zip(arr, ruptures):
+        arrays = surface_to_arrays(rup.surface)  # shape (3, s1, s2)
+        rec['seed'] = rup.rup_id
+        rec['mag'] = rup.mag
+        rec['rake'] = rup.rake
+        rec['lon'] = rup.hypocenter.x
+        rec['lat'] = rup.hypocenter.y
+        rec['dep'] = rup.hypocenter.z
+        rec['multiplicity'] = rup.multiplicity
+        rec['trt'] = rup.tectonic_region_type
+        rec['kind'] = ' '.join(cls.__name__ for cls in code2cls[rup.code])
+        rec['mesh'] = json.dumps(
+            [[[[float5(z) for z in y] for y in x] for x in array]
+             for array in arrays])
+        extra = {}
+        if hasattr(rup, 'probs_occur'):
+            extra['probs_occur'] = rup.probs_occur
+        else:
+            extra['occurrence_rate'] = rup.occurrence_rate
+        if hasattr(rup, 'weight'):
+            extra['weight'] = rup.weight
+        _fixfloat32(extra)
+        rec['extra'] = json.dumps(extra)
+    return arr
+
+
+def from_array(aw):
+    """
+    :returns: a list of ruptures from an ArrayWrapper
+    """
+    rups = []
+    names = aw.array.dtype.names
+    for rec in aw.array:
+        dic = dict(zip(names, rec))
+        dic['trt'] = aw.trts[int(dic.pop('et_id'))]
+        dic['hypo'] = dic.pop('lon'), dic.pop('lat'), dic.pop('dep')
+        dic.update(json.loads(dic.pop('extra')))
+        rups.append(_get_rupture(dic))
+    return rups
+
+
+def _get_rupture(rec, geom=None, trt=None):
+    # rec: a dictionary or a record
+    # geom: if any, an array of floats32 convertible into a mesh
     if not code2cls:
         code2cls.update(BaseRupture.init())
     if geom is None:
-        lons = dic['lons']
-        mesh = numpy.zeros((3, len(lons), len(lons[0])), F32)
-        mesh[0] = dic['lons']
-        mesh[1] = dic['lats']
-        mesh[2] = dic['depths']
-    else:
-        mesh = numpy.zeros((3,) + geom.shape, F32)
-        mesh[0] = geom['lon']
-        mesh[1] = geom['lat']
-        mesh[2] = geom['depth']
-    rupture_cls, surface_cls = code2cls[dic['code']]
-    rupture = object.__new__(rupture_cls)
-    rupture.rup_id = dic['serial']
-    rupture.surface = object.__new__(surface_cls)
-    rupture.mag = dic['mag']
-    rupture.rake = dic['rake']
-    rupture.hypocenter = geo.Point(*dic['hypo'])
-    rupture.occurrence_rate = dic['occurrence_rate']
-    rupture.tectonic_region_type = trt or dic['trt']
+        points = F32([rec['lons'], rec['lats'], rec['depths']]).flat
+        geom = numpy.concatenate([[1], [len(rec['lons']), 1], points])
+
+    # build surface
+    arrays = []
+    num_surfaces = int(geom[0])
+    start = num_surfaces * 2 + 1
+    for i in range(1, 2 * num_surfaces, 2):
+        s1, s2 = int(geom[i]), int(geom[i + 1])
+        size = s1 * s2 * 3
+        array = geom[start:start + size].reshape(3, s1, s2)
+        arrays.append(array)
+        start += size
+    mesh = arrays[0]
+    rupture_cls, surface_cls = code2cls[rec['code']]
+    surface = object.__new__(surface_cls)
     if surface_cls is geo.PlanarSurface:
-        rupture.surface = geo.PlanarSurface.from_array(
-            mesh[:, 0, :])
+        surface = geo.PlanarSurface.from_array(mesh[:, 0, :])
     elif surface_cls is geo.MultiSurface:
-        # mesh has shape (3, n, 4)
-        rupture.surface.__init__([
-            geo.PlanarSurface.from_array(mesh[:, i, :])
-            for i in range(mesh.shape[1])])
+        if all(array.shape == (3, 1, 4) for array in arrays):
+            # for PlanarSurfaces each array has shape (3, 1, 4)
+            surface.__init__([
+                geo.PlanarSurface.from_array(array[:, 0, :])
+                for array in arrays])
+        else:
+            # assume KyteSurfaces
+            surface.__init__([geo.KiteSurface(RectangularMesh(*array))
+                              for array in arrays])
+
     elif surface_cls is geo.GriddedSurface:
         # fault surface, strike and dip will be computed
-        rupture.surface.strike = rupture.surface.dip = None
-        rupture.surface.mesh = Mesh(*mesh)
+        surface.strike = surface.dip = None
+        surface.mesh = Mesh(*mesh)
     else:
         # fault surface, strike and dip will be computed
-        rupture.surface.strike = rupture.surface.dip = None
-        rupture.surface.__init__(RectangularMesh(*mesh))
+        surface.strike = surface.dip = None
+        surface.__init__(RectangularMesh(*mesh))
+
+    # build rupture
+    rupture = object.__new__(rupture_cls)
+    rupture.rup_id = rec['seed']
+    rupture.surface = surface
+    rupture.mag = rec['mag']
+    rupture.rake = rec['rake']
+    rupture.hypocenter = geo.Point(*rec['hypo'])
+    rupture.occurrence_rate = rec['occurrence_rate']
+    try:
+        rupture.probs_occur = rec['probs_occur']
+    except (KeyError, ValueError):  # rec can be a numpy record
+        pass
+    rupture.tectonic_region_type = trt or rec['trt']
+    rupture.multiplicity = rec['n_occ']
     return rupture
 
 
-def from_toml(toml_str):
-    """
-    :param toml_str: a string in TOML format
-    :returns: a rupture instance
-    """
-    return _get_rupture(toml.loads(toml_str))
-
-
 def float5(x):
     # a float with 5 digits
     return round(float(x), 5)
@@ -111,18 +183,17 @@ def _fixfloat32(dic):
         elif isinstance(v, tuple):
             dic[k] = [float5(x) for x in v]
         elif isinstance(v, numpy.ndarray):
-            dic[k] = [[float5(y) for y in x] for x in v]
+            if len(v.shape) == 3:  # 3D array
+                dic[k] = [[[float5(z) for z in y] for y in x] for x in v]
+            elif len(v.shape) == 2:  # 2D array
+                dic[k] = [[float5(y) for y in x] for x in v]
+            elif len(v.shape) == 1:  # 1D array
+                dic[k] = [float5(x) for x in v]
+            else:
+                raise NotImplementedError
 
 
-def to_toml(rup):
-    """
-    :param rup: a rupture instance
-    :returns: a TOML string
-    """
-    return toml.dumps(rup.todict())
-
-
-def to_checksum(cls1, cls2):
+def to_checksum8(cls1, cls2):
     """
     Convert a pair of classes into a numeric code (uint8)
     """
@@ -173,13 +244,15 @@ class BaseRupture(metaclass=abc.ABCMeta):
             general.gen_subclasses(BaseRupture))
         surface_classes = list(general.gen_subclasses(BaseSurface))
         code2cls = {}
+        BaseRupture.str2code = {}
         for rup, sur in itertools.product(rupture_classes, surface_classes):
-            chk = to_checksum(rup, sur)
+            chk = to_checksum8(rup, sur)
             if chk in code2cls and code2cls[chk] != (rup, sur):
                 raise ValueError('Non-unique checksum %d for %s, %s' %
                                  (chk, rup, sur))
             cls._code[rup, sur] = chk
             code2cls[chk] = rup, sur
+            BaseRupture.str2code['%s %s' % (rup.__name__, sur.__name__)] = chk
         return code2cls
 
     def __init__(self, mag, rake, tectonic_region_type, hypocenter,
@@ -200,28 +273,17 @@ class BaseRupture(metaclass=abc.ABCMeta):
         """Returns the code (integer in the range 0 .. 255) of the rupture"""
         return self._code[self.__class__, self.surface.__class__]
 
+    def size(self):
+        """
+        Dummy method for compatibility with the RuptureContext.
+
+        :returns: 1
+        """
+        return 1
+
     get_probability_no_exceedance = (
         contexts.RuptureContext.get_probability_no_exceedance)
 
-    def todict(self):
-        """
-        :returns: a representation of the rupture as a dict
-        """
-        if not code2cls:
-            code2cls.update(BaseRupture.init())
-        hypo = self.hypocenter.x, self.hypocenter.y, self.hypocenter.z
-        mesh = surface_to_array(self.surface)  # shape (3, sy, sz)
-        sy, sz = mesh.shape[1:]
-        dic = {'serial': int(self.rup_id),
-               'mag': self.mag, 'rake': self.rake, 'hypo': hypo,
-               'trt': self.tectonic_region_type,
-               'code': self.code, 'occurrence_rate': self.occurrence_rate,
-               'rupture_cls': self.__class__.__name__,
-               'surface_cls': self.surface.__class__.__name__,
-               'lons': mesh[0], 'lats': mesh[1], 'depths': mesh[2]}
-        _fixfloat32(dic)
-        return dic
-
     def sample_number_of_occurrences(self, n=1):
         """
         Randomly sample number of occurrences from temporal occurrence model
@@ -272,8 +334,7 @@ class NonParametricProbabilisticRupture(BaseRupture):
             mag, rake, tectonic_region_type, hypocenter, surface,
             rupture_slip_direction, weight)
         # an array of probabilities with sum 1
-        self.probs_occur = numpy.array(
-            [prob for (prob, occ) in pmf.data], numpy.float32)
+        self.probs_occur = numpy.array([prob for (prob, occ) in pmf.data])
         self.occurrence_rate = numpy.nan
 
     def sample_number_of_occurrences(self, n=1):
@@ -498,16 +559,20 @@ class ParametricProbabilisticRupture(BaseRupture):
 
 class PointSurface:
     """
-    A fake surface used in PointRuptures
+    A fake surface used in PointRuptures.
+    The parameters `hypocenter`, `strike` and `dip` are determined by
+    collapsing the corresponding parameters in the original PointSource.
     """
-    def __init__(self, hypocenter):
+    def __init__(self, hypocenter, strike, dip):
         self.hypocenter = hypocenter
+        self.strike = strike
+        self.dip = dip
 
     def get_strike(self):
-        return 0
+        return self.strike
 
     def get_dip(self):
-        return 0
+        return self.dip
 
     def get_top_edge_depth(self):
         return self.hypocenter.depth
@@ -527,15 +592,17 @@ class PointRupture(ParametricProbabilisticRupture):
     A rupture coming from a far away PointSource, so that the finite
     size effects can be neglected.
     """
-    def __init__(self, mag, tectonic_region_type, hypocenter,
-                 occurrence_rate, temporal_occurrence_model):
-        self.mag = mag
-        self.rake = 0
+    def __init__(self, mag, tectonic_region_type, hypocenter, strike,
+                 dip, rake, occurrence_rate, temporal_occurrence_model):
         self.tectonic_region_type = tectonic_region_type
         self.hypocenter = hypocenter
+        self.mag = mag
+        self.strike = strike
+        self.rake = rake
+        self.dip = dip
         self.occurrence_rate = occurrence_rate
         self.temporal_occurrence_model = temporal_occurrence_model
-        self.surface = PointSurface(hypocenter)
+        self.surface = PointSurface(hypocenter, strike, dip)
         self.weight = None  # no mutex
 
 
@@ -627,78 +694,57 @@ class ExportedRupture(object):
         self.indices = indices
 
 
-def get_eids(rup_array, samples_by_grp, num_rlzs_by_grp):
-    """
-    :param rup_array: a composite array with fields rup_id, n_occ and grp_id
-    :param samples_by_grp: a dictionary grp_id -> samples
-    :param num_rlzs_by_grp: a dictionary grp_id -> num_rlzs
-    """
-    all_eids = []
-    for rup in rup_array:
-        grp_id = rup['grp_id']
-        samples = samples_by_grp[grp_id]
-        num_rlzs = num_rlzs_by_grp[grp_id]
-        num_events = rup['n_occ'] if samples > 1 else rup['n_occ'] * num_rlzs
-        eids = numpy.arange(num_events, dtype=U32)
-        all_eids.append(eids)
-    return numpy.concatenate(all_eids)
-
-
 class EBRupture(object):
     """
     An event based rupture. It is a wrapper over a hazardlib rupture
     object, containing an array of site indices affected by the rupture,
     as well as the IDs of the corresponding seismic events.
     """
-    def __init__(self, rupture, srcidx, grp_id, n_occ, samples=1, id=None):
+    def __init__(self, rupture, source_id, et_id, n_occ, id=None, e0=0):
         # NB: when reading an exported ruptures.xml the rup_id will be 0
         # for the first rupture; it used to be the seed instead
         assert rupture.rup_id >= 0  # sanity check
         self.rupture = rupture
-        self.srcidx = srcidx
-        self.grp_id = grp_id
+        self.source_id = source_id
+        self.et_id = et_id
         self.n_occ = n_occ
-        self.samples = samples
-        self.id = id  # id of the rupture on the DataStore, to be overridden
+        self.id = id  # id of the rupture on the DataStore
+        self.e0 = e0
+        self.scenario = False
 
     @property
     def rup_id(self):
         """
-        Serial number of the rupture
+        Seed of the rupture
         """
         return self.rupture.rup_id
 
     def get_eids_by_rlz(self, rlzs_by_gsim):
         """
-        :param n_occ: number of occurrences
         :params rlzs_by_gsim: a dictionary gsims -> rlzs array
-        :param samples: number of samples in current source group
         :returns: a dictionary rlz index -> eids array
         """
-        j = 0
         dic = {}
-        if self.samples == 1:  # full enumeration or akin to it
-            for rlzs in rlzs_by_gsim.values():
-                for rlz in rlzs:
-                    dic[rlz] = numpy.arange(j, j + self.n_occ, dtype=U32)
-                    j += self.n_occ
-        else:  # associated eids to the realizations
-            rlzs = numpy.concatenate(list(rlzs_by_gsim.values()))
-            assert len(rlzs) == self.samples, (len(rlzs), self.samples)
+        rlzs = numpy.concatenate(list(rlzs_by_gsim.values()))
+        if self.scenario:
+            all_eids = numpy.arange(self.n_occ, dtype=U32) + self.e0
+            splits = numpy.array_split(all_eids, len(rlzs))
+            for rlz_id, eids in zip(rlzs, splits):
+                dic[rlz_id] = eids
+        else:
+            j = 0
             histo = general.random_histogram(
-                self.n_occ, self.samples, self.rup_id)
+                self.n_occ, len(rlzs), self.rup_id)
             for rlz, n in zip(rlzs, histo):
-                dic[rlz] = numpy.arange(j, j + n, dtype=U32)
+                dic[rlz] = numpy.arange(j, j + n, dtype=U32) + self.e0
                 j += n
         return dic
 
-    def get_eids(self, num_rlzs):
+    def get_eids(self):
         """
-        :param num_rlzs: the number of realizations for the given group
         :returns: an array of event IDs
         """
-        num_events = self.n_occ if self.samples > 1 else self.n_occ * num_rlzs
-        return numpy.arange(num_events, dtype=U32)
+        return numpy.arange(self.n_occ, dtype=U32)
 
     def export(self, events_by_ses):
         """
@@ -754,12 +800,11 @@ class RuptureProxy(object):
 
     :param rec: a record with the rupture parameters
     :param nsites: approx number of sites affected by the rupture
-    :param samples: how many times the rupture is sampled
     """
-    def __init__(self, rec, nsites=None, samples=1):
+    def __init__(self, rec, nsites=None, scenario=False):
         self.rec = rec
         self.nsites = nsites
-        self.samples = samples
+        self.scenario = scenario
 
     @property
     def weight(self):
@@ -768,21 +813,20 @@ class RuptureProxy(object):
             heuristic weight for the underlying rupture, depending on the
             number of occurrences, number of samples and number of sites
         """
-        return self.samples * self['n_occ'] * (
+        return self['n_occ'] * (
             100 if self.nsites is None else max(self.nsites, 100))
 
     def __getitem__(self, name):
         return self.rec[name]
 
     # NB: requires the .geom attribute to be set
-    def to_ebr(self, trt, samples):
+    def to_ebr(self, trt):
         """
         :returns: EBRupture instance associated to the underlying rupture
         """
         # not implemented: rupture_slip_direction
         rupture = _get_rupture(self.rec, self.geom, trt)
-        ebr = EBRupture(rupture, self.rec['srcidx'], self.rec['grp_id'],
-                        self.rec['n_occ'], samples)
-        ebr.id = self.rec['id']
-        ebr.e0 = self.rec['e0']
+        ebr = EBRupture(rupture, self.rec['source_id'], self.rec['et_id'],
+                        self.rec['n_occ'], self.rec['id'], self.rec['e0'])
+        ebr.scenario = self.scenario
         return ebr
