@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2020 GEM Foundation
+# Copyright (C) 2012-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -21,8 +21,6 @@ Module :mod:`openquake.hazardlib.tom` contains implementations of probability
 density functions for earthquake temporal occurrence modeling.
 """
 import abc
-import math
-
 import numpy
 import scipy.stats
 
@@ -85,6 +83,7 @@ class BaseTOM(metaclass=abc.ABCMeta):
         given in the constructor.
         """
 
+
 class FatedTOM(BaseTOM):
 
     def __init__(self, time_span, occurrence_rate=None):
@@ -121,7 +120,7 @@ class PoissonTOM(BaseTOM):
         :return:
             Float value between 0 and 1 inclusive.
         """
-        return 1 - math.exp(- occurrence_rate * self.time_span)
+        return 1 - numpy.exp(- occurrence_rate * self.time_span)
 
     def get_probability_n_occurrences(self, occurrence_rate, num):
         """
@@ -171,7 +170,7 @@ class PoissonTOM(BaseTOM):
         """
         The probability is computed using the following formula ::
 
-            (1 - e ** (-occurrence_rate * time_span)) ** poes
+            e ** (-occurrence_rate * time_span * poes)
 
         :param occurrence_rate:
             The average number of events per year.
@@ -183,8 +182,7 @@ class PoissonTOM(BaseTOM):
             calling the :func:`func <openquake.hazardlib.gsim.base.get_poes>`.
         :return:
             2D numpy array containing probabilities of no exceedance. First
-            dimension represents sites, second dimensions intensity measure
-            levels.
+￼            dimension represents sites, second dimension intensity measure
+￼            levels.
         """
-        p = self.get_probability_one_or_more_occurrences(occurrence_rate)
-        return (1 - p) ** poes
+        return numpy.exp(- occurrence_rate * self.time_span * poes)
