@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2020 GEM Foundation
+# Copyright (C) 2012-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -22,21 +22,22 @@ This module includes the scientific API of the oq-risklib
 import abc
 import copy
 import bisect
-import warnings
 import itertools
 import collections
 from functools import lru_cache
 
 import numpy
+import pandas
 from numpy.testing import assert_equal
 from scipy import interpolate, stats, random
 
-from openquake.baselib.general import CallableDict, cached_property
+from openquake.baselib.general import CallableDict, AccumDict
 from openquake.hazardlib.stats import compute_stats2
 
 F64 = numpy.float64
 F32 = numpy.float32
 U32 = numpy.uint32
+U16 = numpy.uint16
 
 
 def pairwise(iterable):
@@ -122,36 +123,50 @@ class VulnerabilityFunction(object):
         else:
             self.covs = numpy.zeros(self.imls.shape)
 
+        anycovs = self.covs.any()
         for lr, cov in zip(self.mean_loss_ratios, self.covs):
-            if lr == 0.0 and cov > 0.0:
-                msg = ("It is not valid to define a loss ratio = 0.0 with a "
-                       "corresponding coeff. of variation > 0.0")
+            if lr == 0 and cov > 0:
+                msg = ("It is not valid to define a mean loss ratio = 0 "
+                       "with a corresponding coefficient of variation > 0")
                 raise ValueError(msg)
+            if cov < 0:
+                raise ValueError(
+                    'Found a negative coefficient of variation in %s' %
+                    self.covs)
             if distribution == 'BT':
-                if lr > 1:
-                    raise ValueError('The meanLRs must be <= 1, got %s' % lr)
+                if lr == 0:  # possible with cov == 0
+                    pass
+                elif lr > 1:
+                    raise ValueError('The meanLRs must be ≤ 1, got %s' % lr)
+                elif cov == 0 and anycovs:
+                    raise ValueError(
+                        'Found a zero coefficient of variation in %s' %
+                        self.covs)
                 elif cov ** 2 > 1 / lr - 1:
                     # see https://github.com/gem/oq-engine/issues/4841
                     raise ValueError(
-                        'The coefficient of variation %s > %s is too large '
+                        'The coefficient of variation %s > %s does not '
+                        'satisfy the requirement 0 < σ < sqrt[μ × (1 - μ)] '
                         'in %s' % (cov, numpy.sqrt(1 / lr - 1), self))
 
         self.distribution_name = distribution
 
     def init(self):
         # called by CompositeRiskModel and by __setstate__
-        self.stddevs = self.covs * self.mean_loss_ratios
+        self.covs = F64(self.covs)
+        self.mean_loss_ratios = F64(self.mean_loss_ratios)
+        self._stddevs = self.covs * self.mean_loss_ratios
         self._mlr_i1d = interpolate.interp1d(self.imls, self.mean_loss_ratios)
         self._covs_i1d = interpolate.interp1d(self.imls, self.covs)
         self.set_distribution(None)
 
     def set_distribution(self, epsilons=None):
         if (self.covs > 0).any():
-            self.distribution = DISTRIBUTIONS[self.distribution_name]()
+            self._distribution = DISTRIBUTIONS[self.distribution_name]()
         else:
-            self.distribution = DegenerateDistribution()
-        self.distribution.epsilons = (numpy.array(epsilons)
-                                      if epsilons is not None else None)
+            self._distribution = DegenerateDistribution()
+        self._distribution.epsilons = (numpy.array(epsilons)
+                                       if epsilons is not None else None)
         assert self.seed is not None, self
         numpy.random.seed(self.seed)  # set by CompositeRiskModel.init
 
@@ -171,7 +186,7 @@ class VulnerabilityFunction(object):
 
     def sample(self, means, covs, idxs, epsilons=None):
         """
-        Sample the epsilons and apply the corrections to the means.
+        Sample the distribution and apply the corrections to the means.
         This method is called only if there are nonzero covs.
 
         :param means:
@@ -187,8 +202,9 @@ class VulnerabilityFunction(object):
         """
         if self.distribution_name == 'LN' and epsilons is None:
             return means
-        self.set_distribution(epsilons)
-        res = self.distribution.sample(means, covs, means * covs, idxs)
+        elif self.distribution_name == 'LN':
+            self.set_distribution(epsilons)
+        res = self._distribution.sample(means, covs, means * covs, idxs)
         return res
 
     # this is used in the tests, not in the engine code base
@@ -298,8 +314,8 @@ class VulnerabilityFunction(object):
         lrem = numpy.empty((len(loss_ratios), len(self.imls)))
         for row, loss_ratio in enumerate(loss_ratios):
             for col, (mean_loss_ratio, stddev) in enumerate(
-                    zip(self.mean_loss_ratios, self.stddevs)):
-                lrem[row, col] = self.distribution.survival(
+                    zip(self.mean_loss_ratios, self._stddevs)):
+                lrem[row, col] = self._distribution.survival(
                     loss_ratio, mean_loss_ratio, stddev)
         return lrem
 
@@ -320,23 +336,6 @@ class VulnerabilityFunction(object):
             [numpy.mean(pair) for pair in pairwise(self.imls)] +
             [self.imls[-1] + (self.imls[-1] - self.imls[-2]) / 2.])
 
-    def __toh5__(self):
-        """
-        :returns: a pair (array, attrs) suitable for storage in HDF5 format
-        """
-        array = numpy.zeros(len(self.imls), self.dtype)
-        array['iml'] = self.imls
-        array['loss_ratio'] = self.mean_loss_ratios
-        array['cov'] = self.covs
-        return array, {'id': self.id, 'imt': self.imt,
-                       'distribution_name': self.distribution_name}
-
-    def __fromh5__(self, array, attrs):
-        vars(self).update(attrs)
-        self.imls = array['iml']
-        self.mean_loss_ratios = array['loss_ratio']
-        self.covs = array['cov']
-
     def __repr__(self):
         return '<VulnerabilityFunction(%s, %s)>' % (self.id, self.imt)
 
@@ -367,7 +366,7 @@ class VulnerabilityFunctionWithPMF(VulnerabilityFunction):
         self.init()
 
         ls = [('iml', F32)] + [('prob-%s' % lr, F32) for lr in loss_ratios]
-        self.dtype = numpy.dtype(ls)
+        self._dtype = numpy.dtype(ls)
 
     def init(self):
         # the seed is reset in CompositeRiskModel.__init__
@@ -375,9 +374,9 @@ class VulnerabilityFunctionWithPMF(VulnerabilityFunction):
         self.set_distribution(None)
 
     def set_distribution(self, epsilons=None):
-        self.distribution = DISTRIBUTIONS[self.distribution_name]()
-        self.distribution.epsilons = epsilons
-        self.distribution.seed = self.seed  # needed only for PM
+        self._distribution = DISTRIBUTIONS[self.distribution_name]()
+        self._distribution.epsilons = epsilons
+        self._distribution.seed = self.seed  # needed only for PM
 
     def __getstate__(self):
         return (self.id, self.imt, self.imls, self.loss_ratios,
@@ -432,7 +431,7 @@ class VulnerabilityFunctionWithPMF(VulnerabilityFunction):
            array of E' probabilities
         """
         self.set_distribution(epsilons)
-        return self.distribution.sample(self.loss_ratios, probs)
+        return self._distribution.sample(self.loss_ratios, probs)
 
     @lru_cache()
     def loss_ratio_exceedance_matrix(self, loss_ratios):
@@ -448,29 +447,11 @@ class VulnerabilityFunctionWithPMF(VulnerabilityFunction):
         # TODO: to be implemented if the classical risk calculator
         # needs to support the pmf vulnerability format
 
-    def __toh5__(self):
-        """
-        :returns: a pair (array, attrs) suitable for storage in HDF5 format
-        """
-        array = numpy.zeros(len(self.imls), self.dtype)
-        array['iml'] = self.imls
-        for i, lr in enumerate(self.loss_ratios):
-            array['prob-%s' % lr] = self.probs[i]
-        return array, {'id': self.id, 'imt': self.imt,
-                       'distribution_name': self.distribution_name}
-
-    def __fromh5__(self, array, attrs):
-        lrs = [n.split('-')[1] for n in array.dtype.names if '-' in n]
-        self.loss_ratios = map(float, lrs)
-        self.imls = array['iml']
-        self.probs = array
-        vars(self).update(attrs)
-
     def __repr__(self):
         return '<VulnerabilityFunctionWithPMF(%s, %s)>' % (self.id, self.imt)
 
 
-# this is meant to be instantiated by riskmodels.get_risk_models
+# this is meant to be instantiated by riskmodels.get_risk_functions
 class VulnerabilityModel(dict):
     """
     Container for a set of vulnerability functions. You can access each
@@ -496,17 +477,25 @@ class VulnerabilityModel(dict):
 # ############################## fragility ############################### #
 
 class FragilityFunctionContinuous(object):
-    # FIXME (lp). Should be re-factored with LogNormalDistribution
-    def __init__(self, limit_state, mean, stddev):
+
+    def __init__(self, limit_state, mean, stddev, minIML, maxIML, nodamage=0):
         self.limit_state = limit_state
         self.mean = mean
         self.stddev = stddev
+        self.minIML = minIML
+        self.maxIML = maxIML
+        self.no_damage_limit = nodamage
 
     def __call__(self, imls):
         """
         Compute the Probability of Exceedance (PoE) for the given
         Intensity Measure Levels (IMLs).
         """
+        # it is essentially to make a copy of the intensity measure levels,
+        # otherwise the minIML feature in continuous fragility functions will
+        # change the levels, thus breaking case_master for OQ_DISTRIBUTE=no
+        if self.minIML or self.maxIML:
+            imls = numpy.array(imls)
         variance = self.stddev ** 2.0
         sigma = numpy.sqrt(numpy.log(
             (variance / self.mean ** 2.0) + 1.0))
@@ -514,11 +503,14 @@ class FragilityFunctionContinuous(object):
         mu = self.mean ** 2.0 / numpy.sqrt(
             variance + self.mean ** 2.0)
 
-        return stats.lognorm.cdf(imls, sigma, scale=mu)
-
-    def __getstate__(self):
-        return dict(limit_state=self.limit_state,
-                    mean=self.mean, stddev=self.stddev)
+        if self.maxIML:
+            imls[imls > self.maxIML] = self.maxIML
+        if self.minIML:
+            imls[imls < self.minIML] = self.minIML
+        result = stats.lognorm.cdf(imls, sigma, scale=mu)
+        if self.no_damage_limit:
+            result[imls < self.no_damage_limit] = 0
+        return result
 
     def __repr__(self):
         return '<%s(%s, %s, %s)>' % (
@@ -600,36 +592,33 @@ class FragilityFunctionList(list):
         :returns: a populated FragilityFunctionList instance
         """
         new = copy.copy(self)
+        new.clear()
         add_zero = (self.format == 'discrete' and
                     self.nodamage and self.nodamage <= self.imls[0])
         new.imls = build_imls(new, discretization)
         if steps_per_interval > 1:
-            new.interp_imls = build_imls(  # passed to classical_damage
+            new._interp_imls = build_imls(  # passed to classical_damage
                 new, discretization, steps_per_interval)
         for i, ls in enumerate(limit_states):
             data = self.array[i]
             if self.format == 'discrete':
                 if add_zero:
+                    if len(self.imls) == len(data):  # add no_damage
+                        imls = [self.nodamage] + self.imls
+                    else:  # already added
+                        imls = self.imls
                     new.append(FragilityFunctionDiscrete(
-                        ls, [self.nodamage] + self.imls,
-                        numpy.concatenate([[0.], data]),
+                        ls, imls, numpy.concatenate([[0.], data]),
                         self.nodamage))
                 else:
                     new.append(FragilityFunctionDiscrete(
                         ls, self.imls, data, self.nodamage))
             else:  # continuous
                 new.append(FragilityFunctionContinuous(
-                    ls, data['mean'], data['stddev']))
+                    ls, data[0], data[1],  # mean and stddev
+                    self.minIML, self.maxIML, self.nodamage))
         return new
 
-    def __toh5__(self):
-        return self.array, {k: v for k, v in vars(self).items()
-                            if k != 'array' and v is not None}
-
-    def __fromh5__(self, array, attrs):
-        self.array = array
-        vars(self).update(attrs)
-
     def __repr__(self):
         kvs = ['%s=%s' % item for item in vars(self).items()]
         return '<FragilityFunctionList %s>' % ', '.join(kvs)
@@ -715,23 +704,6 @@ class FragilityModel(dict):
             self.__class__.__name__, self.lossCategory,
             self.limitStates, sorted(self))
 
-    def build(self, continuous_fragility_discretization, steps_per_interval):
-        """
-        Return a new FragilityModel instance, in which the values have been
-        replaced with FragilityFunctionList instances.
-
-        :param continuous_fragility_discretization:
-            configuration parameter
-        :param steps_per_interval:
-            configuration parameter
-        """
-        newfm = copy.copy(self)
-        for key, ffl in self.items():
-            newfm[key] = ffl.build(self.limitStates,
-                                   continuous_fragility_discretization,
-                                   steps_per_interval)
-        return newfm
-
 
 #
 # Distribution & Sampling
@@ -782,23 +754,22 @@ class DegenerateDistribution(Distribution):
 
 def make_epsilons(matrix, seed, correlation):
     """
-    Given a matrix N * R returns a matrix of the same shape N * R
+    Given a matrix of shape (A, E) returns a matrix of the same shape
     obtained by applying the multivariate_normal distribution to
-    N points and R samples, by starting from the given seed and
+    A points and E samples, by starting from the given seed and
     correlation.
     """
     if seed is not None:
         numpy.random.seed(seed)
-    asset_count = len(matrix)
-    samples = len(matrix[0])
+    A = len(matrix)
+    E = len(matrix[0])
     if not correlation:  # avoid building the covariance matrix
-        return numpy.random.normal(size=(samples, asset_count)).transpose()
-    means_vector = numpy.zeros(asset_count)
-    covariance_matrix = (
-        numpy.ones((asset_count, asset_count)) * correlation +
-        numpy.diag(numpy.ones(asset_count)) * (1 - correlation))
+        return numpy.random.normal(size=(E, A)).transpose()
+    means_vector = numpy.zeros(A)
+    covariance_matrix = (numpy.ones((A, A)) * correlation +
+                         numpy.diag(numpy.ones(A)) * (1 - correlation))
     return numpy.random.multivariate_normal(
-        means_vector, covariance_matrix, samples).transpose()
+        means_vector, covariance_matrix, E).transpose()
 
 
 @DISTRIBUTIONS.add('LN')
@@ -838,13 +809,34 @@ class LogNormalDistribution(Distribution):
         return stats.lognorm.sf(loss_ratio, sigma, scale=mu)
 
 
+# NB: the beta distribution `numpy.random.beta(alpha, beta)` is singular
+# if the beta array contains some zeros; this happens if the vulnerability
+# function has zero coefficients of variation (stddevs).
+# Even if you do something like this:
+#
+# res = numpy.zeros_like(alpha)
+# ok = beta !=0  # not singular
+# res[ok] = numpy.random.beta(alpha[ok], beta[ok])
+# res[~ok] = 1
+#
+# this is not going to give results close to you want expect by
+# setting stddev=.0000001 and mean=.0000001 (i.e. smoothly going
+# through the limit) even if the the seed is fixed. The reason is that
+# the random number generator will advance differently.  Suppose the
+# array size is 10 and there is a single singular value with beta=0
+# and 9 values with beta != 0; the call to numpy.random.beta(alpha, beta)
+# will advance the generator by 9 steps, while if you regularize the
+# singularity by using stddev=.0000001 and mean=.00000001 the random
+# generator will advance by 10 steps. The numbers produced by the beta
+# distribution will be quite different.
+# This is why having stddevs == 0 is an error and it is forbidden in
+# VulnerabilityFunction.__init__.
 @DISTRIBUTIONS.add('BT')
 class BetaDistribution(Distribution):
     def sample(self, means, _covs, stddevs, _idxs=None):
         alpha = self._alpha(means, stddevs)
         beta = self._beta(means, stddevs)
-        res = numpy.random.beta(alpha, beta, size=None)
-        return res
+        return numpy.random.beta(alpha, beta)
 
     def survival(self, loss_ratio, mean, stddev):
         return stats.beta.sf(loss_ratio,
@@ -865,13 +857,13 @@ class DiscreteDistribution(Distribution):
     seed = None  # to be set
 
     def sample(self, loss_ratios, probs):
-        ret = []
+        ret = numpy.zeros(probs.shape[1])
         r = numpy.arange(len(loss_ratios))
         for i in range(probs.shape[1]):
             random.seed(self.seed + i)
             # the seed is set inside the loop to avoid block-size dependency
             pmf = stats.rv_discrete(name='pmf', values=(r, probs[:, i])).rvs()
-            ret.append(loss_ratios[pmf])
+            ret[i] = loss_ratios[pmf]
         return ret
 
     def survival(self, loss_ratios, probs):
@@ -926,15 +918,15 @@ def annual_frequency_of_exceedence(poe, t_haz):
     :param t_haz: hazard investigation time
     :returns: array of frequencies (with +inf values where poe=1)
     """
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore")
-        # avoid RuntimeWarning: divide by zero encountered in log
-        return - numpy.log(1. - poe) / t_haz
+    arr = 1. - poe
+    arr[arr == 0] = 1E-16  # cutoff to avoid log(0)
+    return - numpy.log(arr) / t_haz
 
 
 def classical_damage(
         fragility_functions, hazard_imls, hazard_poes,
-        investigation_time, risk_investigation_time):
+        investigation_time, risk_investigation_time,
+        steps_per_interval=1, debug=False):
     """
     :param fragility_functions:
         a list of fragility functions for each damage state
@@ -946,13 +938,14 @@ def classical_damage(
         hazard investigation time
     :param risk_investigation_time:
         risk investigation time
+    :param steps_per_interval:
+        steps per interval
     :returns:
         an array of M probabilities of occurrence where M is the numbers
         of damage states.
     """
-    spi = fragility_functions.steps_per_interval
-    if spi and spi > 1:  # interpolate
-        imls = numpy.array(fragility_functions.interp_imls)
+    if steps_per_interval > 1:  # interpolate
+        imls = numpy.array(fragility_functions._interp_imls)
         min_val, max_val = hazard_imls[0], hazard_imls[-1]
         assert min_val > 0, hazard_imls  # sanity check
         numpy.putmask(imls, imls < min_val, min_val)
@@ -966,11 +959,10 @@ def classical_damage(
         pairwise_mean([afe[0]] + list(afe) + [afe[-1]]))
     poes_per_damage_state = []
     for ff in fragility_functions:
-        frequency_of_exceedence_per_damage_state = numpy.dot(
-            annual_frequency_of_occurrence, list(map(ff, imls)))
-        poe_per_damage_state = 1. - numpy.exp(
-            - frequency_of_exceedence_per_damage_state *
-            risk_investigation_time)
+        fx = annual_frequency_of_occurrence @ ff(imls)
+        if debug:
+            print(fx)
+        poe_per_damage_state = 1. - numpy.exp(-fx * risk_investigation_time)
         poes_per_damage_state.append(poe_per_damage_state)
     poos = pairwise_diff([1] + poes_per_damage_state + [0])
     return poos
@@ -1017,6 +1009,7 @@ def classical(vulnerability_function, hazard_imls, hazard_poes, loss_ratios):
     return numpy.array([loss_ratios, lrem_po.sum(axis=1)])
 
 
+# used in classical_risk only
 def conditional_loss_ratio(loss_ratios, poes, probability):
     """
     Return the loss ratio corresponding to the given PoE (Probability
@@ -1308,46 +1301,54 @@ def losses_by_period(losses, return_periods, num_events=None, eff_time=None):
     """
     :param losses: array of simulated losses
     :param return_periods: return periods of interest
-    :param num_events: the number of events (>= to the number of losses)
+    :param num_events: the number of events (>= number of losses)
     :param eff_time: investigation_time * ses_per_logic_tree_path
     :returns: interpolated losses for the return periods, possibly with NaN
 
     NB: the return periods must be ordered integers >= 1. The interpolated
     losses are defined inside the interval min_time < time < eff_time
     where min_time = eff_time /num_events. On the right of the interval they
-    have NaN values and on the left zero values. Here is an example:
+    have NaN values; on the left zero values.
+    If num_events is not passed, it is inferred from the number of losses;
+    if eff_time is not passed, it is inferred from the longest return period.
+    Here is an example:
 
     >>> losses = [3, 2, 3.5, 4, 3, 23, 11, 2, 1, 4, 5, 7, 8, 9, 13]
     >>> losses_by_period(losses, [1, 2, 5, 10, 20, 50, 100], 20)
     array([ 0. ,  0. ,  0. ,  3.5,  8. , 13. , 23. ])
-
-    If num_events is not passed, it is inferred from the number of losses;
-    if eff_time is not passed, it is inferred from the longest return period.
     """
     P = len(return_periods)
-    if len(losses) == 0:  # zero-curve
-        return numpy.zeros(P)
+    assert len(losses)
+    if isinstance(losses, list):
+        losses = numpy.array(losses)
+    shp = losses.shape[:-1]  # total shape is (L...E)
+    num_losses = losses.shape[-1]
     if num_events is None:
-        num_events = len(losses)
-    elif num_events < len(losses):
+        num_events = num_losses
+    elif num_events < num_losses:
         raise ValueError(
-            'There are not enough events (%d) to compute the loss curve'
-            % num_events)
+            'There are not enough events (%d<%d) to compute the loss curve'
+            % (num_events, num_losses))
     if eff_time is None:
         eff_time = return_periods[-1]
     losses = numpy.sort(losses)
-    num_zeros = num_events - len(losses)
+    # num_losses < num_events: just add zeros
+    num_zeros = num_events - num_losses
     if num_zeros:
-        losses = numpy.concatenate(
-            [numpy.zeros(num_zeros, losses.dtype), losses])
+        newlosses = numpy.zeros(shp + (num_events,), losses.dtype)
+        newlosses[..., num_events-num_losses:num_events] = losses
+        losses = newlosses
     periods = eff_time / numpy.arange(num_events, 0., -1)
     num_left = sum(1 for rp in return_periods if rp < periods[0])
     num_right = sum(1 for rp in return_periods if rp > periods[-1])
     rperiods = [rp for rp in return_periods if periods[0] <= rp <= periods[-1]]
-    curve = numpy.zeros(len(return_periods))
-    c = numpy.interp(numpy.log(rperiods), numpy.log(periods), losses)
-    curve[num_left:P-num_right] = c
-    curve[P-num_right:] = numpy.nan
+    curve = numpy.zeros(shp + (len(return_periods),), losses.dtype)
+    logr, logp = numpy.log(rperiods), numpy.log(periods)
+    for idx, _ in numpy.ndenumerate(losses[..., 0]):
+        tup = idx + (slice(num_left, P-num_right),)
+        curve[tup] = numpy.interp(logr, logp, losses[idx])
+        tup = idx + (slice(P-num_right, None),)
+        curve[tup] = numpy.nan
     return curve
 
 
@@ -1387,33 +1388,6 @@ class LossCurvesMapsBuilder(object):
             array_stats = None
         return array, array_stats
 
-    # used in event_based_risk postproc
-    def build(self, losses_by_event, stats=()):
-        """
-        :param losses_by_event:
-            the aggregate loss table with shape R -> (E, L)
-        :param stats:
-            list of pairs [(statname, statfunc), ...]
-        :returns:
-            two arrays with shape (P, R, L) and (P, S, L)
-        """
-        P, R = len(self.return_periods), len(self.weights)
-        L = len(self.loss_dt.names)
-        array = numpy.zeros((P, R, L), F32)
-        for r in losses_by_event:
-            num_events = self.num_events.get(r, 0)
-            losses = losses_by_event[r]
-            for l, lt in enumerate(self.loss_dt.names):
-                ls = losses[:, l].flatten()  # flatten only in ucerf
-                # NB: do not use squeeze or the gmf_ebrisk tests will break
-                try:
-                    lbp = losses_by_period(
-                        ls, self.return_periods, num_events, self.eff_time)
-                except ValueError as exc:
-                    raise exc.__class__('%s for %s, rlz=' % (exc, lt, r))
-                array[:, r, l] = lbp
-        return self.pair(array, stats)
-
     # used in event_based_risk
     def build_curve(self, asset_value, loss_ratios, rlzi):
         return asset_value * losses_by_period(
@@ -1438,101 +1412,121 @@ class LossCurvesMapsBuilder(object):
                         array[a, r, c, lti] = clratio
         return self.pair(array, stats)
 
-    # used in ebrisk
-    def build_curves(self, loss_arrays, rlzi):
-        if len(loss_arrays) == 0:
-            return ()
-        shp = loss_arrays[0].shape  # (L, T...)
-        P = len(self.return_periods)
-        curves = numpy.zeros((P,) + shp, F32)
-        num_events = self.num_events.get(rlzi, 0)
-        acc = collections.defaultdict(list)
-        for loss_array in loss_arrays:
-            for idx, loss in numpy.ndenumerate(loss_array):
-                acc[idx].append(loss)
-        for idx, losses in acc.items():
-            curves[(slice(None),) + idx] = losses_by_period(
-                losses, self.return_periods, num_events, self.eff_time)
-        return curves
-
-    def gen_curves_by_rlz(self, losses_by_event, ses_ratio):
-        """
-        :param losses_by_event: a dataframe
-        :param ses_ratio: ses ratio
-        :yield: triples (rlzi, curves, losses)
-        """
-        for rlzi, losses_df in losses_by_event.groupby('rlzi'):
-            losses = numpy.array(losses_df)
-            yield (rlzi, self.build_curves(losses, rlzi),
-                   losses.sum(axis=0) * ses_ratio)
+    # used in post_risk
+    def build_curves(self, losses, rlzi):
+        return losses_by_period(
+            losses, self.return_periods, self.num_events[rlzi], self.eff_time)
 
 
-class LossesByAsset(object):
+class AggLossTable(AccumDict):
     """
-    A class to compute losses by asset.
-
-    :param assetcol: an AssetCollection instance
-    :param policy_name: the name of the policy field (can be empty)
-    :param policy_dict: dict loss_type -> array(deduct, limit) (can be empty)
+    A dictionary of matrices of shape L', with L' the total number of loss
+    types (primary + secondary).
+    :param aggkey: a dictionary tuple -> integer
+    :param loss_types: a list of primary loss types
+    :param sec_losses: a list of SecondaryLosses (can be empty)
     """
-    alt = None  # set by the ebrisk calculator
-    losses_by_E = None  # set by the ebrisk calculator
+    @classmethod
+    def new(cls, aggkey, loss_types, sec_losses=()):
+        self = cls()
+        self.aggkey = {key: k for k, key in enumerate(aggkey)}
+        self.aggkey[()] = len(aggkey)
+        self.loss_names = list(loss_types)
+        self.sec_losses = sec_losses
+        for sec_loss in sec_losses:
+            self.loss_names.extend(sec_loss.outputs)
+        self.accum = numpy.zeros(len(self.loss_names), F32)
+        return self
 
-    @cached_property
-    def losses_by_A(self):
+    def aggregate(self, out, minimum_loss, aggby):
         """
-        :returns: an array of shape (A, L)
+        Populate the event loss table
         """
-        return numpy.zeros((self.A, len(self.loss_names)), F32)
+        eids = out.eids
+        assets = out.assets
 
-    def __init__(self, assetcol, loss_names, policy_name='', policy_dict={}):
-        self.A = len(assetcol)
+        # initialize secondary losses outputs, if any
+        for sec_loss in self.sec_losses:
+            for k in sec_loss.outputs:
+                setattr(out, k, numpy.zeros((len(assets), len(eids))))
+
+        # populate outputs
+        if aggby == ['id']:
+            idxs = [self.aggkey[o1, ] for o1 in assets['ordinal'] + 1]
+        elif aggby == ['site_id']:
+            idxs = [self.aggkey[s1, ] for s1 in assets['site_id'] + 1]
+        elif aggby:
+            idxs = [self.aggkey[tuple(rec)] for rec in assets[aggby]]
+        else:
+            idxs = []
+        for a, asset in enumerate(out.assets):
+            lt_losses = []
+            for lti, lt in enumerate(out.loss_types):
+                ls = out[lt][a]
+                if minimum_loss[lt]:
+                    ls[ls < minimum_loss[lt]] = 0
+                lt_losses.append((lt, ls))
+
+            # secondary outputs, if any
+            for sec_loss in self.sec_losses:
+                for k, o in sec_loss.compute(asset, lt_losses, eids).items():
+                    out[k][a] = o
+
+        # aggregation
+        K = len(self.aggkey) - 1
+        for lni, ln in enumerate(self.loss_names):
+            for eid, loss in zip(eids, out[ln].T):
+                self[eid, K][lni] += loss.sum()
+            # this is the slow part, if aggregate_by is given
+            for asset, idx, losses in zip(assets, idxs, out[ln]):
+                for eid, loss in zip(eids, losses):
+                    if loss:
+                        self[eid, idx][lni] += loss
+
+    def to_dframe(self):
+        """
+        Convert the AggLosTable into a DataFrame
+        """
+        out = AccumDict(accum=[])  # col -> values
+        for (eid, idx), arr in self.items():
+            out['event_id'].append(eid)
+            out['agg_id'].append(idx)
+            for l, ln in enumerate(self.loss_names):
+                out[ln].append(arr[l])
+        out['event_id'] = U32(out['event_id'])
+        out['agg_id'] = U32(out['agg_id'])
+        for ln in self.loss_names:
+            out[ln] = F32(out[ln])
+        return pandas.DataFrame(out)
+
+
+# must have attribute .outputs and method .compute(asset, losses, loss_type)
+# returning a dictionary sec_key -> sec_losses
+class InsuredLosses(object):
+    """
+    There is an insured loss for each loss type in the policy dictionary.
+    """
+    def __init__(self, policy_name, policy_dict):
         self.policy_name = policy_name
         self.policy_dict = policy_dict
-        self.loss_names = loss_names
-        self.lni = {ln: i for i, ln in enumerate(loss_names)}
+        self.outputs = [lt + '_ins' for lt in policy_dict]
 
-    def gen_losses(self, out):
+    def compute(self, asset, lt_losses, eids):
         """
-        :yields: pairs (loss_name_index, losses array of shape (A, E))
+        :param asset: an asset record
+        :param lt_losses: a list of pairs (loss_type, E losses)
+        :param eids: an array of E event IDs
+        :returns: a dictionary loss_type_ins -> E insured losses
         """
-        for lt in out.loss_types:
-            lratios = out[lt]  # shape (A, E)
-            losses = numpy.zeros_like(lratios)
-            avalues = (out.assets['occupants_None'] if lt == 'occupants'
-                       else out.assets['value-' + lt])
-            for a, avalue in enumerate(avalues):
-                losses[a] = avalue * lratios[a]
-            yield self.lni[lt], losses  # shape (A, E)
+        res = {}
+        policy_idx = asset[self.policy_name]
+        for lt, losses in lt_losses:
             if lt in self.policy_dict:
-                ins_losses = numpy.zeros_like(lratios)
-                for a, asset in enumerate(out.assets):
-                    ded, lim = self.policy_dict[lt][asset[self.policy_name]]
-                    ins_losses[a] = insured_losses(
-                        losses[a], ded * avalues[a], lim * avalues[a])
-                yield self.lni[lt + '_ins'], ins_losses
-
-    def aggregate(self, out, eidx, minimum_loss, tagidxs, ws):
-        """
-        Populate .losses_by_A, .losses_by_E and .alt
-        """
-        numlosses = numpy.zeros(2, int)
-        for lni, losses in self.gen_losses(out):
-            if ws is not None:  # compute avg_losses, really fast
-                aids = out.assets['ordinal']
-                self.losses_by_A[aids, lni] += losses @ ws
-            self.losses_by_E[eidx, lni] += losses.sum(axis=0)
-            if tagidxs is not None:
-                # this is the slow part, depending on minimum_loss
-                for a, asset in enumerate(out.assets):
-                    idx = ','.join(map(str, tagidxs[a]))
-                    kept = 0
-                    for loss, eid in zip(losses[a], out.eids):
-                        if loss >= minimum_loss[lni]:
-                            self.alt[idx][eid][lni] += loss
-                            kept += 1
-                    numlosses += numpy.array([kept, len(losses[a])])
-        return numlosses
+                avalue = asset['value-' + lt]
+                ded, lim = self.policy_dict[lt][policy_idx]
+                res[lt + '_ins'] = insured_losses(
+                    losses, ded * avalue, lim * avalue)
+        return res
 
 
 # ####################### Consequences ##################################### #
