@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2015-2020 GEM Foundation
+# Copyright (C) 2015-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -20,7 +20,6 @@ import getpass
 import requests
 import logging
 import django
-import numpy
 
 from time import sleep
 from django.conf import settings
@@ -32,10 +31,13 @@ if settings.LOCKDOWN:
 
 
 def is_superuser(request):
-    if settings.LOCKDOWN and hasattr(request, 'user'):
-        if request.user.is_superuser:
-            return True
-    return False
+    """
+    Without authentication (settings.LOCKDOW is false) every user is considered
+    a superuser, otherwise look at the attribute `request.user.is_superuser`.
+    """
+    if not settings.LOCKDOWN:
+        return True
+    return request.user.is_superuser if hasattr(request, 'user') else False
 
 
 def get_user(request):
@@ -133,29 +135,3 @@ def check_webserver_running(url="http://localhost:8800", max_retries=30):
         logging.warning('Unable to connect to %s within %s retries'
                         % (url, max_retries))
     return success
-
-
-def array_of_strings_to_bytes(arr, key):
-    """
-    :param arr: array or array-like object
-    :param key: string associated to the error (appear in the error message)
-
-    If `arr` is a numpy array with dtype object containing strings, convert
-    it into a numpy array containing bytes, unless it has more than 2
-    dimensions or contains non-strings (these are errors). Return `arr`
-    unchanged in the other cases.
-    """
-    if arr is None:
-        return ()
-    if not isinstance(arr, numpy.ndarray) or arr.dtype != numpy.dtype('O'):
-        return arr
-    if arr.ndim == 1:
-        return numpy.array([s.encode('utf8') for s in arr])
-    elif arr.ndim == 2:
-        return numpy.array([[col.encode('utf8') for col in row]
-                            for row in arr])
-    else:
-        raise NotImplementedError('The array for %s has shape %s' %
-                                  (key, arr.shape))
-
-    return arr
